package com.gct.aoid.algorithm;

import java.util.LinkedList;
import java.util.TreeMap;

/**
 * dec:
 * <p>
 * Melkman
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2022/1/10 下午5:54
 */
public class Melkman {
    private final Point[] pointArray;//坐标数组
    private final int N;//数据个数
    private final int D[]; // 数组索引

    public Melkman(Double[][] array) {
        N = array.length;
        this.pointArray = new Point[N];
        D = new int[2 * N];
        for (int i = 0; i < N; i++)
            pointArray[i] = new Point(i, array[i][0], array[i][1]);
    }

    /**
     * 求凸包点	 * 	 * @return 所求凸包点
     */
    public Double[][] convexHullPoint() {        // 获得最小的Y，作为P0点
        double minY = pointArray[0].y;
        int j = 0;
        for (int i = 1; i < N; i++) {
            if (pointArray[i].y < minY) {
                minY = pointArray[i].y;
                j = i;
            }
        }
        swap(0, j);        // 计算除第一顶点外的其余顶点到第一点的线段与x轴的夹角
        for (int i = 1; i < N; i++) {
            pointArray[i].arCos = angle(i);
        }
        quickSort(1, N - 1); // 根据所得到的角度进行快速排序
        int bot = N - 1;
        int top = N;
        D[top++] = 0;
        D[top++] = 1;
        int i;
        for (i = 2; i < N; i++) {// 寻找第三个点 要保证3个点不共线！！
            if (isLeft(pointArray[D[top - 2]], pointArray[D[top - 1]], pointArray[i]) != 0) break;
            D[top - 1] = i; // 共线就更换顶点
        }
        D[bot--] = i;
        D[top++] = i; // i是第三个点 不共线！！
        int t;
        if (isLeft(pointArray[D[N]], pointArray[D[N + 1]], pointArray[D[N + 2]]) < 0) {            // 此时队列中有3个点，要保证3个点a,b,c是成逆时针的，不是就调换ab
            t = D[N];
            D[N] = D[N + 1];
            D[N + 1] = t;
        }
        for (i++; i < N; i++) {            // 如果成立就是i在凸包内，跳过 //top=n+3 bot=n-2
            if (isLeft(pointArray[D[top - 2]], pointArray[D[top - 1]], pointArray[i]) > 0 && isLeft(pointArray[D[bot + 1]], pointArray[D[bot + 2]], pointArray[i]) > 0) {
                continue;
            }            //非左转 则退栈
            while (isLeft(pointArray[D[top - 2]], pointArray[D[top - 1]], pointArray[i]) <= 0) {
                top--;
            }
            D[top++] = i;            //反向表非左转 则退栈
            while (isLeft(pointArray[D[bot + 1]], pointArray[D[bot + 2]], pointArray[i]) <= 0) {
                bot++;
            }
            D[bot--] = i;
        }
        // 凸包构造完成，D数组里bot+1至top-1内就是凸包的序列(头尾是同一点)
    /*    TreeMap<Integer, Double[]> resultPoints = new TreeMap<>();
        for (i = bot + 1; i < top - 1; i++) {
            Point point = pointArray[D[i]];
            resultPoints.put(point.index, new Double[]{point.x, point.y});
        }
*/
        LinkedList<Double[]> resultPoints = new LinkedList<>();
        for (i = bot + 1; i < top - 1; i++) {
            Point point = pointArray[D[i]];
            resultPoints.add(new Double[]{point.x, point.y});
        }
        resultPoints.add(new Double[]{resultPoints.getFirst()[0],resultPoints.getFirst()[1]});//闭环
        return resultPoints.toArray(new Double[][]{});
    }

    /**
     * 判断ba相对ao是不是左转	 * 	 * @return 大于0则左转
     */
    private double isLeft(Point o, Point a, Point b) {
        double aoX = a.x - o.x;
        double aoY = a.y - o.y;
        double baX = b.x - a.x;
        double baY = b.y - a.y;
        return aoX * baY - aoY * baX;
    }

    /**
     * 实现数组交换	 * 	 * @param i	 * @param j
     */
    private void swap(int i, int j) {
        Point tempPoint = pointArray[j];
        pointArray[j] = pointArray[i];
        pointArray[i] = tempPoint;
    }

    /**
     * 快速排序	 * 	 * @param top	 * @param bot
     */
    private void quickSort(int top, int bot) {
        int pos;
        if (top < bot) {
            pos = loc(top, bot);
            quickSort(top, pos - 1);
            quickSort(pos + 1, bot);
        }
    }

    /**
     * 移动起点，左侧为小，右侧为大	 * 	 * @param top	 * @param bot	 * @return 移动后的位置
     */
    private int loc(int top, int bot) {
        double x = pointArray[top].arCos;
        int j, k;
        j = top + 1;
        k = bot;
        while (true) {
            while (j < bot && pointArray[j].arCos < x) j++;
            while (k > top && pointArray[k].arCos > x) k--;
            if (j >= k) break;
            swap(j, k);
        }
        swap(top, k);
        return k;
    }

    /**
     * 角度计算	 * 	 * @param i 指针	 * @return
     */
    private double angle(int i) {
        double j, k, m, h;
        j = pointArray[i].x - pointArray[0].x;
        k = pointArray[i].y - pointArray[0].y;
        m = Math.sqrt(j * j + k * k); // 得到顶点i 到第一顶点的线段长度
        if (k < 0) j = (-1) * Math.abs(j);
        h = Math.acos(j / m); // 得到该线段与x轴的角度
        return h;
    }


}

class Point {
    int index;//原数组中的索引
    double x;        //X坐标
    double y;        //Y坐标
    double arCos;    //与P0点的角度

    public Point(int index, double x, double y) {
        this.index = index;
        this.x = x;
        this.y = y;
    }
}