package com.gct.aoid.algorithm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

public class PointFind {

    //判断点p在直线y=ax+b的哪一侧,-1左上，1右下，0在线上,p1为直线右上点，p2为左下点
    private int category(double[] p1, double[] p2, double[] p) {
        double res = (p1[0] - p[0]) * (p2[1] - p[1]) - (p1[1] - p[1]) * (p2[0] - p[0]);
        if (res > 0) return 1;
        if (res < 0) return -1;
        return 0;
    }

    public double[] getPointLeftUp(double[][] gt,double a) {
        double l = -1000, r = 1000;//使用y=x+b进行第一次逼近
        double[] ans = new double[]{0,0};
        while (r-l>1e-6) {
            double b = (l+r)/2;
            List<double[]> leftUpPointList = new ArrayList<>();
            List<double[]>rightDownPointList = new ArrayList<>();
            for (int i=0;i<gt.length;i++){
                int category = category(new double[]{1000,a*1000+b}, new double[]{0,0+b}, gt[i]);
                if (category == -1) {
                    leftUpPointList.add(gt[i]);
                }else{
                    rightDownPointList.add(gt[i]);
                }
            }
//            if (leftUpPointList.size()>=1){  //找到角点
//                ans = leftUpPointList.get(0);
//                l=b;
//            }else {
//
//                r=b;
//            }
            if (rightDownPointList.size()>=1){
                ans = rightDownPointList.get(0);
                r=b;
            }
            else l=b;
        }
        return ans;
    }

    public double calEffectStroke(double[][] sgt,String diagnoseResult2){
        double[][] gt = BezierInterpolation(sgt,sgt.length,2);
        double k = (diagnoseResult2.contains("气影响") || diagnoseResult2.contains("供液不足"))
                && ! diagnoseResult2.contains("轻微")
                ?
                1.5:3.0;
        if(diagnoseResult2.contains("气影响") || diagnoseResult2.contains("供液不足")){
            double[] maxLoad = Arrays.stream(gt).max(Comparator.comparingDouble(x -> x[1])).get();
            double[] minLoad = Arrays.stream(gt).min(Comparator.comparingDouble(x -> x[1])).get();
            double limit = minLoad[1] + (maxLoad[1] - minLoad[1])/3.5;
            gt = Arrays.stream(gt).filter(x->x[1]<limit).collect(Collectors.toList()).toArray(new double[][]{});
            k=10;
        }
        double[] res = getPointLeftUp(gt,k);
        return res[0];
    }

    public double[][] BezierInterpolation(double[][] poss, int precision, int dimersion) {
        //double[][] bezier_points = new double[0][];
        List<double[]> bezier_points = new ArrayList<>();
        //维度，坐标轴数（二维坐标，三维坐标...）
        if (dimersion != 2) return bezier_points.toArray(new double[][]{});
        //贝塞尔曲线控制点数（阶数），此说法不太对，但是便于理解记忆
        int number = poss.length;
        //控制点数不小于 2 ，至少为二维坐标系
        if (number < 2 || dimersion < 2)
            return bezier_points.toArray(new double[][]{});

        //double[][] result = new double[precision][];
        //计算杨辉三角
        double[] mi = new double[number];
        mi[0] = mi[1] = 1;
        for (int i = 3; i <= number; i++) {
            double[] t = new double[(i - 1)];
            System.arraycopy(mi, 0, t, 0, i - 1);
            /*for (int j = 0; j < i - 1; j++) {
                t[j] = mi[j];
            }*/
            mi[0] = mi[i - 1] = 1;
            for (int j = 0; j < i - 2; j++) {
                mi[j + 1] = t[j] + t[j + 1];
            }
        }

        //计算坐标点
        for (int i = 0; i < precision; i++) {
            double t = (double) i / precision;
            double[] temp = new double[dimersion];
            temp[0] = temp[1] = 0;
            for (int j = 0; j < dimersion; j++) {
                for (int k = 0; k < number; k++) {
                    temp[j] += Math.pow(1 - t, number - k - 1) * poss[k][j] * Math.pow(t, k) * mi[k];
                    //幂运算改为循环乘，防止溢出???
                    //var c_temp=poss[k][j]* mi[k];
                    //for (var c = 1; c <= number - k - 1; c++)
                    //    c_temp *= (1 - t);
                    //for (var c = 1; c <= k; c++)
                    //    c_temp *= t;
                    //temp += c_temp;
                }
            }
            bezier_points.add(temp);
        }
        return bezier_points.toArray(new double[][]{});
    }

}