package com.gct.aoid.config;

import com.gct.common.util.StringUtil;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;


/**
 * feign 调用配置，统一添加token请求头
 *
 * @author: NewMeanning
 * @create: 2021-01-29 00:10
 **/
@Slf4j
@Configuration
public class FeignConfig implements RequestInterceptor {

    @Value("${auth.hostList}")
    private String tokeAuthUrl;
    private static String token;

    private static String lockStr = "token";

    @Bean(name = "restTemplate")
    RestTemplate getRestTemplate() {
        return new RestTemplate();
    }

    @Override
    public void apply(RequestTemplate requestTemplate) {
        if (StringUtil.isBlankOrEmpty(token)) {
            synchronized (lockStr) {
                if (StringUtil.isBlankOrEmpty(token)) {
                    List<String> hostList = Arrays.asList(tokeAuthUrl.split(","));
                    //手动获取RestTemplate 实例
                    AnnotationConfigApplicationContext applicationContext = new AnnotationConfigApplicationContext();
                    applicationContext.register(RestTemplate.class);
                    applicationContext.refresh();
                    RestTemplate restTemplate = (RestTemplate) applicationContext.getBean("restTemplate");
                    //自定义轮询方式负载
                    int index = ThreadLocalRandom.current().nextInt(hostList.size());
                    String url = "http://" + hostList.get(index) + "/token";
                    token = restTemplate.getForObject(url, String.class);
                    log.info("token:{}", token);
                }
            }
        }
        requestTemplate.header("token", token);//请求头添加token

    }




}