package com.gct.aoid.config;

import com.gct.aoid.job.*;
import com.gct.aoid.job.properties.*;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;
import java.text.ParseException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;

import static org.quartz.CronScheduleBuilder.cronSchedule;

/**
 * class name: QuartzTaskConfig
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/21 上午10:21
 */
@Configuration
@EnableConfigurationProperties(MutableJobConfigProperties.class)
public class QuartzTaskConfig {
    private static final ch.qos.logback.classic.Logger reload = (ch.qos.logback.classic.Logger) LoggerFactory.getLogger("reload");

    private static final   String jobName = "JobMonitor";

    private static final   String jobGroup = "JOBMONITOR_GROUP";

    private static final  String jobName2 = "CoefficientCorrection";
    private static final  String jobGroup2 = "CoefficientCorrectionJob_Group";


    @Autowired
    private MutableJobConfigProperties jobConfigProperties;



    @Bean
    public JobDetail jobDetail1() {
        JobConfigurationProperties config = jobConfigProperties.getCalDailyLiqJob();
        JobParamConfigurationProperties param = config.getParam();
        try {
            return JobBuilder.newJob(CalLiqDailyJobDetail.class)
                    .withIdentity(param.getJobName(), param.getJobGroup())
//                    .usingJobData(JobDataKey.JobName.getDec(), triggerConfigProperties.getJt1().getJobName())
//                    .usingJobData(JobDataKey.JobGroup.getDec(), triggerConfigProperties.getJt1().getJobGroup())
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", param.getJobName(), param.getJobGroup(), e.getMessage());
            throw e;
        }
    }

    @Bean
    public JobDetail jobMonitor() {
        try {
            return JobBuilder.newJob(JobMonitor.class)
                    .withIdentity(jobName, jobGroup)
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", jobGroup, jobGroup);
            throw e;
        }
    }

    @Bean
    public JobDetail TimingSummaryJob() {
        JobConfigurationProperties config = jobConfigProperties.getCalDailyLiqJob();
        JobParamConfigurationProperties param = config.getParam();
        try {
            return JobBuilder.newJob(TimingSummaryJob.class)//
                    .withIdentity(param.getJobName()+"-Correct", param.getJobGroup()+"-Correct")
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", jobGroup, jobGroup);
            throw e;
        }
    }


    @Bean
    public Trigger trigger1() {
        JobConfigurationProperties config = jobConfigProperties.getCalDailyLiqJob();
        JobParamConfigurationProperties param = config.getParam();
        TriggerConfigurationProperties trigger = config.getTrigger();
        try {
            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInHours(1);
            if (trigger.isRepeatable())
                scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob(param.getJobName(), param.getJobGroup())
                    .startAt(new Date(new Date().getTime() + 5000))
//                    .startNow()
                    .withSchedule(scheduleBuilder).build();
        } catch (Exception e) {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}", param.getJobName(), param.getJobGroup(), e.getMessage());
            throw e;
        }
    }
    @Bean
    public Trigger trigger2() throws ParseException {
        JobConfigurationProperties config = jobConfigProperties.getCalDailyLiqJob();
        JobParamConfigurationProperties param = config.getParam();
        LocalDateTime startDate = LocalDateTime.parse(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))
                + " " + param.getStartTimeEveryDay(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        try {
            if (startDate.isBefore(LocalDateTime.now())){
                startDate = startDate.plusDays(1);
            }
            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInSeconds(60*60*24);
            scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob(param.getJobName(), param.getJobGroup())
                    .startAt(Date.from(startDate.atZone(ZoneId.systemDefault()).toInstant()))
                    .withSchedule(scheduleBuilder).build();
        } catch (Exception e) {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}", param.getJobName(), param.getJobGroup(), e.getMessage());
            throw e;
        }
    }

    /***
     * 监控CalLiqDailyJobDetail计算报警
     * */
    @Bean
    public Trigger trigger3() throws IOException {
        try {
            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInSeconds(60*10);
            scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob(jobName, jobGroup)
                    .startAt(new Date(new Date().getTime() ))
                    .withSchedule(scheduleBuilder).build();
        } catch (Exception e) {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}",jobName, jobGroup, e.getMessage());
            throw e;
        }
    }

    /***
     * 每天重新汇总计算
     * */
    @Bean
    public Trigger trigger4() throws IOException {
        JobConfigurationProperties config = jobConfigProperties.getCalDailyLiqJob();
        JobParamConfigurationProperties param = config.getParam();
        try {
            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInSeconds(60);
            scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob(param.getJobName()+"-Correct", param.getJobGroup()+"-Correct")
                    .startNow()
                    .withSchedule(scheduleBuilder).build();
        } catch (Exception e) {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}",param.getJobName()+"-Correct", param.getJobGroup()+"-Correct", e.getMessage());
            throw e;
        }
    }


    /**
     * 量液系数校正服务调度JobDetail
     * @return
     */
    @Bean
    public JobDetail CoefficientCorrectionJob() {

        CoefficientCorrectionJobProperties property = jobConfigProperties.getCoefficientCorrectionJob();
        try {
            return JobBuilder.newJob(CoefficientCorrectionJob.class)
                    .withIdentity(property.getJobName(), property.getJobGroup())
                    .usingJobData(JobDataKey.JobName.getDec(), property.getJobName())
                    .usingJobData(JobDataKey.JobGroup.getDec(), property.getJobGroup())
                    .storeDurably()
                    .build();
        }catch (Exception e)
        {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", property.getJobGroup(), property.getJobGroup());
            throw e;
        }

    }

    /**
     * 量液系数校正服务触发器
     * @return
     */
    @Bean
    public Trigger CoefficientCorrectionJobTrigger() throws IOException{
        CoefficientCorrectionJobProperties property = jobConfigProperties.getCoefficientCorrectionJob();
        try {
            return TriggerBuilder.newTrigger()
                    .forJob(property.getJobName(), property.getJobGroup())
                    .withSchedule(cronSchedule(property.getCronStr())).build();
        }catch (Exception e)
        {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}",property.getJobName(),  property.getJobGroup(), e.getMessage());
            throw e;
        }


    }



    /**
     * 误差分析服务调度JobDetail
     * @return
     */
    @Bean
    public JobDetail YieldErrorAnalysisJob() {

        YieldErrorAnalysisJobProperties property = jobConfigProperties.getYieldErrorAnalysisJob();
        try {
            return JobBuilder.newJob(YieldErrorAnalysisJob.class)
                    .withIdentity(property.getJobName(), property.getJobGroup())
                    .usingJobData(JobDataKey.JobName.getDec(), property.getJobName())
                    .usingJobData(JobDataKey.JobGroup.getDec(), property.getJobGroup())
                    .storeDurably()
                    .build();
        }catch (Exception e)
        {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", property.getJobGroup(), property.getJobGroup());
            throw e;
        }

    }

    /**
     * 误差分析服务触发器
     * @return
     */
    @Bean
    public Trigger YieldErrorAnalysisJobTrigger() throws IOException{
        YieldErrorAnalysisJobProperties property = jobConfigProperties.getYieldErrorAnalysisJob();
        try {
            return TriggerBuilder.newTrigger()
                    .forJob(property.getJobName(), property.getJobGroup())
                    .withSchedule(cronSchedule(property.getCronStr())).build();
        }catch (Exception e)
        {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}",property.getJobName(),  property.getJobGroup(), e.getMessage());
            throw e;
        }


    }


    /**
     * 每天自动插入默认的量液汇总数据
     */
    //@Bean
    public Trigger insertDefultAoidDailyDataJobTrigger(){
        try {
            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInSeconds(60);
            scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob("InsertDefultAoidDailyData", "INSERTDEFULTAOIDDAILYDATAGROUP")
                    .startNow()
                    .withSchedule(scheduleBuilder).build();
        } catch (Exception e) {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}","InsertDefultAoidDailyData"
                    , "INSERTDEFULTAOIDDAILYDATAGROUP", e.getMessage());
            throw e;
        }
    }

    /**
     * 每天自动插入默认的量液汇总数据
     */
   // @Bean
    public JobDetail insertDefultAoidDailyDataJob() {
        try {
            return JobBuilder.newJob(insertDefultAoidDailyData.class)
                    .withIdentity("InsertDefultAoidDailyData", "INSERTDEFULTAOIDDAILYDATAGROUP")
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", "InsertDefultAoidDailyData", "INSERTDEFULTAOIDDAILYDATAGROUP",e.getMessage());
            throw e;
        }
    }




    /**
     * 稀油标定系数任务
     * @return
     */
//    @Bean
    public JobDetail CalibrationParameterJob() {

        CalibrationParameterJobProperties property = jobConfigProperties.getCalibrationParameterJob();
        try {
            return JobBuilder.newJob(CalibrationParameterJob.class)
                    .withIdentity(property.getJobName(), property.getJobGroup())
                    .usingJobData(JobDataKey.JobName.getDec(), property.getJobName())
                    .usingJobData(JobDataKey.JobGroup.getDec(), property.getJobGroup())
                    .storeDurably()
                    .build();
        }catch (Exception e)
        {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", property.getJobGroup(), property.getJobGroup());
            throw e;
        }

    }

    /**
     * 稀油标定系数任务触发器
     * @return
     */
//    @Bean
    public Trigger CalibrationParameterJobTrigger() throws IOException{
        CalibrationParameterJobProperties property = jobConfigProperties.getCalibrationParameterJob();
        try {
            return TriggerBuilder.newTrigger()
                    .forJob(property.getJobName(), property.getJobGroup())
                    .withSchedule(cronSchedule(property.getCronStr())).build();
        }catch (Exception e)
        {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}",property.getJobName(),  property.getJobGroup(), e.getMessage());
            throw e;
        }


    }


}
