package com.gct.aoid.job;

import com.gct.aoid.entity.AoidParamSetting;
import com.gct.aoid.job.properties.JobConfigurationProperties;
import com.gct.aoid.job.properties.JobParamConfigurationProperties;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.service.AoidLiquidAbnormalService;
import com.gct.aoid.service.AoidParamSettingService;
import com.gct.common.util.DateTimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * class name: calAoidLiqError
 *
 * @author gxt
 * @version 1.0
 * @since 2021/12/22 下午7:07 周三
 */
@Slf4j
public class CalAoidLiqError extends QuartzJobBean {
    @Autowired
    AoidLiquidAbnormalService aoidLiquidAbnormalService;
    @Autowired
    AoidParamSettingService aoidParamSettingService;
    @Autowired
    private MutableJobConfigProperties jobConfigProperties;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        JobConfigurationProperties config = jobConfigProperties.getCalDailyLiqJob();
        JobParamConfigurationProperties param = config.getParam();
        String jobName = "calAoidLiqError";
        String jobGroup = "CALAOIDLIQERROR_GROUP";
        log.info("*===>[quartz-job:start]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        LocalTime LocalTime = java.time.LocalTime.now();
        try {
            if (param.getCalLiqDate().equals(LocalTime.format(DateTimeFormatter.ofPattern("HH:mm")))) {
                List<AoidParamSetting> aoidParamSetting = aoidParamSettingService.getAoidParamSetting();
                LocalDateTime localDateTime = DateTimeUtil.formatDateTime(LocalDate.now().minusDays(1) + " 00:00:00");
                boolean b = aoidLiquidAbnormalService.addAoidLiquidAbnormalNew(localDateTime, aoidParamSetting);
                if (!b)
                    log.error("[quartz-job:run error]JobName:{},JobGroup:{}\n-----------cause by:{}", jobName, jobGroup, "insert error");
            }
        } catch (Exception e) {
            log.error("[quartz-job:run error]JobName:{},JobGroup:{}\n-----------cause by:{}", jobName, jobGroup, e.getStackTrace());
            e.printStackTrace();
        }
        log.info("*===>[quartz-job:completed]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
    }
}
