package com.gct.aoid.job;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gct.aoid.entity.log.LogFile;
import com.gct.aoid.job.properties.JobParamConfigurationProperties;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.service.AoidErrorReloadService;
import com.gct.aoid.service.LiqProdDailyCalService;
import com.gct.aoid.utils.LogFileUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.locks.AbstractQueuedSynchronizer;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.StampedLock;


/**
 * class name: CalLiqDailyJobDetail
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/21 上午10:24
 */
@Slf4j
@EnableConfigurationProperties(MutableJobConfigProperties.class)
public class CalLiqDailyJobDetail extends QuartzJobBean {

    private static ReentrantLock lock = new ReentrantLock();
    @Autowired
    public LiqProdDailyCalService service;

    @Autowired
    AoidErrorReloadService aoidErrorReloadService;

    @Autowired
    MutableJobConfigProperties config;

    @SneakyThrows
    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {

        //sumDataTaskMapper= (SumDataTaskMapper)jobExecutionContext.getJobDetail().getJobDataMap().get("dao");
        JobParamConfigurationProperties param = config.getCalDailyLiqJob().getParam();
        String jobName = param.getJobName();
        String jobGroup = param.getJobGroup();
        log.info("*===>[quartz-job:start]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        //region job content
        if (!lock.tryLock()) return;
        else {
            try {
                if (param.isRunning()) {
                    TimingSummaryJob.threadLocal.set(true);
                    List<LogFile> logFiles = LogFileUtils.getLogFile();
                    LogFile logFile;
                    if (!Objects.isNull(logFiles) && logFiles.size() > 0) {
                        logFile = logFiles.get(logFiles.size() - 1);
                        String s = param.getStartDate() + " " + param.getStartTimeEveryDay();
                        //判断最新应计算时间
                        if (logFile.getProdDate().isAfter(LocalDateTime.parse(s, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")))) {
                            String startDate = logFile.getProdDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
                            service.calculate(logFile.getOrgId(), startDate, param.getStartTimeEveryDay());
                        } else {
                            service.calculate(param.getOrgId(), param.getStartDate(), param.getStartTimeEveryDay());
                        }
                    }
                    logFile = new LogFile().setId(UUID.randomUUID().toString().replace("-", "")).setOrgId(param.getOrgId()).setProdDate(LocalDateTime.now());
                    LogFileUtils.writeLogFile((JSONObject) JSON.toJSON(logFile));
                } else {
                    return;
                }
            } catch (Exception e) {
                log.error("[quartz-job:run error]JobName:{},JobGroup:{}\n-----------cause by:{}", jobName, jobGroup, e.getStackTrace());
                e.printStackTrace();
            } finally {

                TimingSummaryJob.threadLocal.remove();
            }
            lock.unlock();
        }
        //endregion
        log.info("*===>[quartz-job:completed]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
    }


}
