package com.gct.aoid.job;

import com.gct.aoid.job.properties.JobParamConfigurationProperties;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.job.statusEnum.JobStatusEnum;
import com.gct.aoid.service.LiqProdDailyCalService;
import com.gct.aoid.utils.LogFileUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;


/**
 * class name: CalLiqDailyJobDetail
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/21 上午10:24
 */
@Slf4j
@EnableConfigurationProperties(MutableJobConfigProperties.class)
public class JobMonitor extends QuartzJobBean {

    @Autowired
    public LiqProdDailyCalService service;

    @Autowired
    MutableJobConfigProperties config;

    @SneakyThrows
    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        JobParamConfigurationProperties param = config.getCalDailyLiqJob().getParam();
        String jobName = "JobMonitor";
        String jobGroup ="JOBMONITOR_GROUP";
        log.info("*===>[quartz-job:start]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        try {
            boolean running = LogFileUtils.isRunning(LocalDateTime.now());
            if (running) {
                param.setStatus(JobStatusEnum.NORMAL.getStatus());
            }else {
                param.setStatus(JobStatusEnum.RUNTIME_OUT_EXCEPTION.getStatus());
            }
            if (!param.isRunning()) param.setStatus(JobStatusEnum.PAUSE.getStatus());
        } catch (Exception e) {
            log.error("[quartz-job:run error]JobName:{},JobGroup:{}\n-----------cause by:{}", jobName, jobGroup, e.getStackTrace());
            e.printStackTrace();
        }
        //endregion
        log.info("*===>[quartz-job:completed]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
    }



}
