package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.AoidLiquidAbnormal;
import com.gct.aoid.entity.AoidSinglegtYield;
import com.gct.aoid.entity.Vo.*;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.TimingSummaryJob;
import com.gct.aoid.mapper.AoidDailyYieldMapper;
import com.gct.aoid.mapper.AoidLiquidAbnormalMapper;
import com.gct.aoid.mapper.AoidSinglegtYieldMapper;
import com.gct.aoid.service.AoidDailyYieldRefactorService;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidSinglegtYieldService;
import com.gct.aoid.service.LiqProdDailyCalService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.gct.aoid.service.impl.AoidDailyYieldServiceImpl.round2;
import static java.util.stream.Collectors.toList;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jyx
 * @since 2021-01-28
 */
@Service
@Slf4j
public class AoidDailyYieldRefactorServiceImpl extends ServiceImpl<AoidDailyYieldMapper, AoidDailyYield> implements AoidDailyYieldRefactorService {
    private DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    @Autowired
    private AoidDailyYieldMapper aoidDailyYieldMapper;
    @Autowired
    BaseDataClient baseDataClient;
    @Autowired
    private AoidLiquidAbnormalMapper aoidLiquidAbnormalMapper;

    @Override
    public AoidYeildStatVo getAoidDailyYieldStat(AoidYeildVo aoidYeildVo) {
        AoidYeildStatVo aoidYeildStatVo = new AoidYeildStatVo();
        String startDate = aoidYeildVo.getStartDate() + " 00:00:00";
        String endDate = aoidYeildVo.getEndDate() + " 00:00:00";
        List<String> wellids = new ArrayList<>();
        List<String> objects = new ArrayList<>();
        List<String> errorgts = new ArrayList<>();
        if (aoidYeildVo.getWellId()!=null){
            wellids.add(aoidYeildVo.getWellId());
        }else {
             wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
            //wellids.add("WELLT");
            //wellids.add("WELLTL100000492");
            //wellids.add("WELLTL100018711");
        }
        LambdaQueryWrapper<AoidDailyYield> queryWrapper = new LambdaQueryWrapper<>();
        LambdaQueryWrapper<AoidLiquidAbnormal> abnormalLambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (wellids != null && !wellids.isEmpty()) {
            queryWrapper.in(AoidDailyYield::getWellId, wellids);
            abnormalLambdaQueryWrapper.in(AoidLiquidAbnormal::getWellId, wellids);
        }
        queryWrapper.ge(AoidDailyYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(startDate, df))).
                le(AoidDailyYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(endDate, df))).orderByAsc(AoidDailyYield::getProdDate);
        //根据时间+org查询量液数据
        List<AoidDailyYield> aoidDailyYields = aoidDailyYieldMapper.selectList(queryWrapper);
        //获取上月平均产量
         List<AoidYeildStatMonthVo> avP =  aoidDailyYieldMapper.selectAoidDailyYieldLastMonth(wellids);
        //根据时间查询上个有效自然日的数据
        AoidYeildStatMonthVo  aoidDailyYieldLastDay =  aoidDailyYieldMapper.selectAoidDailyYieldLastDay(wellids,LocalDate.parse(aoidYeildVo.getStartDate()));
        //获取一月内上个正常日产量
        double  lastDayTotal = 0;
        if(Objects.nonNull(aoidDailyYieldLastDay)){
            lastDayTotal = aoidDailyYieldLastDay.getLiqProdDailyTotal();
        }
        //量液异常数量
        abnormalLambdaQueryWrapper.ge(AoidLiquidAbnormal::getProdDate, Timestamp.valueOf(LocalDateTime.parse(startDate, df))).
                le(AoidLiquidAbnormal::getProdDate, Timestamp.valueOf(LocalDateTime.parse(endDate, df)));
        aoidYeildStatVo.setLiqProdDailyAbnormalCount(aoidLiquidAbnormalMapper.selectCount(abnormalLambdaQueryWrapper));
        //上月总量液
        double lastMonthCount = 0;
        //上月平均量液
        double avgLastMonthCount = 0;
        for (AoidYeildStatMonthVo aoidYeildStatMonthVo : avP) {
            lastMonthCount+=aoidYeildStatMonthVo.getLiqProdDailyTotal();
        }
        avgLastMonthCount = lastMonthCount/avP.size();
         //当日总产液量
         double  liqProdDailyTotal = 0;
        //当日日志产液量
         double  a2LiqProdDailyTotal = 0;

        //有效功图数
        int yxgtsCount =0;
        //错误功图数
        int errorgtsCount =0;
        for (AoidDailyYield e : aoidDailyYields) {
            if (e.getLiqProdDaily()!=null){
                objects.add(e.getWellId());
                liqProdDailyTotal+=e.getLiqProdDaily();
            }
            if (e.getA2LiqProdDaily()!=null){
                a2LiqProdDailyTotal+=e.getA2LiqProdDaily();
            }
            if (e.getYxgts()!=null){
                yxgtsCount+=e.getYxgts();
            }
            if (e.getErrorgts()!=null&& e.getErrorgts()>0){
                errorgts.add(e.getWellId());
                errorgtsCount+=e.getErrorgts();
            }
        }
        List<String> objectsList = new ArrayList(new HashSet(objects));
        List<String> errorgtsList = new ArrayList(new HashSet(errorgts));
        aoidYeildStatVo.setNotLipWellCount(0);
         if (wellids.size()>objectsList.size()){
            aoidYeildStatVo.setNotLipWellCount(wellids.size()-objectsList.size());
        }
        aoidYeildStatVo.setErrorgtsWellCount(errorgtsList.size());
        //判断产液量比较日志产液量(0.一致1.下降2.上升)
        if (liqProdDailyTotal<a2LiqProdDailyTotal){
            aoidYeildStatVo.setLiqCompare(1);
        }else if (liqProdDailyTotal>a2LiqProdDailyTotal){
            aoidYeildStatVo.setLiqCompare(2);
        }else {
            aoidYeildStatVo.setLiqCompare(0);
        }
        //当日产量与上月平均产量比(0.一致1.下降2.上升)
        double v = liqProdDailyTotal / (liqProdDailyTotal + avgLastMonthCount) - avgLastMonthCount / (liqProdDailyTotal + avgLastMonthCount);
        if (liqProdDailyTotal<avgLastMonthCount){
            aoidYeildStatVo.setAvProductionCompare(1);
            aoidYeildStatVo.setAvProduction(Math.abs(round2(v)));
        }else if (liqProdDailyTotal>avgLastMonthCount){
            aoidYeildStatVo.setAvProductionCompare(2);
            aoidYeildStatVo.setAvProduction(Math.abs(round2(v)));
        }else {
            aoidYeildStatVo.setAvProductionCompare(0);
        }

        //当日产量与上个正常日比(0.一致1.下降2.上升)
        double v1 = liqProdDailyTotal / (liqProdDailyTotal + lastDayTotal) - lastDayTotal / (liqProdDailyTotal + lastDayTotal);
        if (liqProdDailyTotal<lastDayTotal){
            aoidYeildStatVo.setLastDayProductionCompare(1);
            aoidYeildStatVo.setLastDayProduction(Math.abs(round2(v1)));
        }else if (liqProdDailyTotal>lastDayTotal){
            aoidYeildStatVo.setLastDayProductionCompare(2);
            aoidYeildStatVo.setLastDayProduction(Math.abs(round2(v1)));
        }else {
            aoidYeildStatVo.setLastDayProductionCompare(0);
        }
        aoidYeildStatVo.setLiqProdDailyTotal(liqProdDailyTotal);
        aoidYeildStatVo.setA2LiqProdDailyTotal(a2LiqProdDailyTotal);
        aoidYeildStatVo.setYxgtsCount(yxgtsCount);
        aoidYeildStatVo.setErrorgtsCount(errorgtsCount);
        return aoidYeildStatVo;
    }

    @Override
    public List<AoidYeildStatTrendVo> getAoidDailyYieldStatTrend(AoidYeildVo aoidYeildVo) {
         List<String> wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
        /*List<String> wellids = new ArrayList<>();
        wellids.add("WELLT");
        wellids.add("WELLTL100000492");
        wellids.add("WELLTL100018711");*/
        List<AoidYeildStatTrendVo> aoidYeildStatTrendVoList = new ArrayList<>();
        DateTimeFormatter dfd = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        aoidYeildVo.setStartDate(aoidYeildVo.getStartDate() + " 00:00:00");
        aoidYeildVo.setEndDate(aoidYeildVo.getEndDate() + " 00:00:00");
        aoidYeildVo.setWellIdList(wellids);
        List<AoidYeildStatDayVo>  aoidDailyYieldDayList =  aoidDailyYieldMapper.selectAoidDailyYieldDay(aoidYeildVo);
        for (AoidYeildStatDayVo aoidYeildStatDayVo : aoidDailyYieldDayList) {
            AoidYeildStatTrendVo aoidYeildStatTrendVo = new AoidYeildStatTrendVo();
            aoidYeildStatTrendVo.setProdDate(aoidYeildStatDayVo.getProdDate().format(dfd));
            //根据时间查询上个有效自然日的数据
             AoidYeildStatMonthVo  aoidDailyYieldLastDay =  aoidDailyYieldMapper.selectAoidDailyYieldLastDay(wellids,aoidYeildStatDayVo.getProdDate().toLocalDate());
            //当日总产液量
            double  liqProdDailyTotal = 0;
            //当日日志产液量
            double  a2LiqProdDailyTotal = 0;
            if (aoidYeildStatDayVo.getLiqProdDailyTotal()!=null){
                liqProdDailyTotal=aoidYeildStatDayVo.getLiqProdDailyTotal();
            }
            if (aoidYeildStatDayVo.getA2LiqProdDailyTotal()!=null){
                a2LiqProdDailyTotal=aoidYeildStatDayVo.getA2LiqProdDailyTotal();
            }
            //获取一月内上个正常日产量
            double  lastDayTotal = 0;
            if(Objects.nonNull(aoidDailyYieldLastDay)){
                lastDayTotal = aoidDailyYieldLastDay.getLiqProdDailyTotal();
            }
            //判断产液量比较日志产液量(0.一致1.下降2.上升)
            if (liqProdDailyTotal<a2LiqProdDailyTotal){
                aoidYeildStatTrendVo.setLiqCompare(1);
            }else if (liqProdDailyTotal>a2LiqProdDailyTotal){
                aoidYeildStatTrendVo.setLiqCompare(2);
            }else {
                aoidYeildStatTrendVo.setLiqCompare(0);
            }
            //当日产量与上个正常日比(0.一致1.下降2.上升)
            double v1 = liqProdDailyTotal / (liqProdDailyTotal + lastDayTotal) - lastDayTotal / (liqProdDailyTotal + lastDayTotal);
            if (liqProdDailyTotal<lastDayTotal){
                aoidYeildStatTrendVo.setLastDayProductionCompare(1);
                aoidYeildStatTrendVo.setLastDayProduction(Math.abs(round2(v1)));
            }else if (liqProdDailyTotal>lastDayTotal){
                aoidYeildStatTrendVo.setLastDayProductionCompare(2);
                aoidYeildStatTrendVo.setLastDayProduction(Math.abs(round2(v1)));
            }else {
                aoidYeildStatTrendVo.setLastDayProductionCompare(0);
            }
            aoidYeildStatTrendVo.setLiqProdDailyTotal(liqProdDailyTotal);
            aoidYeildStatTrendVo.setA2LiqProdDailyTotal(a2LiqProdDailyTotal);
            aoidYeildStatTrendVoList.add(aoidYeildStatTrendVo);
        }
        return aoidYeildStatTrendVoList;
    }

    @Override
    public List<AoidYeildErrorGtsTrendVo> getAoidDailyYieldStatErrorGtsTrend(AoidYeildVo aoidYeildVo) {
         List<String> wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
        /*List<String> wellids = new ArrayList<>();
        wellids.add("WELLT");
        wellids.add("WELLTL100016432");
        wellids.add("WELLTL100017249");*/
        List<AoidYeildErrorGtsTrendVo> aoidYeildErrorGtsTrendVos = new ArrayList<>();
        DateTimeFormatter dfd = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        aoidYeildVo.setStartDate(aoidYeildVo.getStartDate() + " 00:00:00");
        aoidYeildVo.setEndDate(aoidYeildVo.getEndDate() + " 00:00:00");
        aoidYeildVo.setWellIdList(wellids);
        List<AoidYeildStatDayVo>  aoidDailyYieldDayList =  aoidDailyYieldMapper.selectAoidDailyYieldErrorGtsDay(aoidYeildVo);
        for (AoidYeildStatDayVo aoidYeildStatDayVo : aoidDailyYieldDayList) {
            AoidYeildErrorGtsTrendVo aoidYeildErrorGtsTrendVo = new AoidYeildErrorGtsTrendVo();
            aoidYeildErrorGtsTrendVo.setProdDate(aoidYeildStatDayVo.getProdDate().format(dfd));
            //根据时间查询上个有效自然日的数据
            AoidYeildStatMonthVo  aoidDailyYieldErrorGtsDay =  aoidDailyYieldMapper.selectAoidDailyYieldErrorGtsLastDay(wellids,aoidYeildStatDayVo.getProdDate().toLocalDate());
            //当日错误工图数
            double  errorgtsTotal = 0;
            if (aoidYeildStatDayVo.getErrorgtsTotal()!=null){
                errorgtsTotal=aoidYeildStatDayVo.getErrorgtsTotal();
            }
            //获取一月内上个正常日错误工图数量
            double  lastDayTotal = 0;
            if(Objects.nonNull(aoidDailyYieldErrorGtsDay)){
                lastDayTotal = aoidDailyYieldErrorGtsDay.getErrorgtsTotal();
            }
            //当日产量与上个正常日比(0.一致1.下降2.上升)
            double v1 = errorgtsTotal / (errorgtsTotal + lastDayTotal) - lastDayTotal / (errorgtsTotal + lastDayTotal);
            if (errorgtsTotal<lastDayTotal){
                aoidYeildErrorGtsTrendVo.setLastDayErrorGtsCountCompare(1);
                aoidYeildErrorGtsTrendVo.setLastDayErrorGtsCountProduction(Math.abs(round2(v1)));
            }else if (errorgtsTotal>lastDayTotal){
                aoidYeildErrorGtsTrendVo.setLastDayErrorGtsCountCompare(2);
                aoidYeildErrorGtsTrendVo.setLastDayErrorGtsCountProduction(Math.abs(round2(v1)));
            }else {
                aoidYeildErrorGtsTrendVo.setLastDayErrorGtsCountCompare(0);
            }
            aoidYeildErrorGtsTrendVo.setErrorGtsCount(errorgtsTotal);
            aoidYeildErrorGtsTrendVos.add(aoidYeildErrorGtsTrendVo);
        }
        return aoidYeildErrorGtsTrendVos;
    }

    @Override
    public List<AoidYeildAbnormalTrendVo> getAoidDailyYieldAbnormalTrend(AoidYeildVo aoidYeildVo) {

        List<String> wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
        /*List<String> wellids = new ArrayList<>();
        wellids.add("WELLT");
        wellids.add("WELLTL100000492");
        wellids.add("WELLTL100000727");*/
        List<AoidYeildAbnormalTrendVo> aoidYeildAbnormalTrendVos = new ArrayList<>();
        DateTimeFormatter dfd = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        aoidYeildVo.setStartDate(aoidYeildVo.getStartDate() + " 00:00:00");
        aoidYeildVo.setEndDate(aoidYeildVo.getEndDate() + " 00:00:00");
        aoidYeildVo.setWellIdList(wellids);
        List<AoidYeildStatDayVo>  aoidDailyYieldDayList =  aoidLiquidAbnormalMapper.selectAoidDailyYieldAbnormalDay(aoidYeildVo);
        for (AoidYeildStatDayVo aoidYeildStatDayVo : aoidDailyYieldDayList) {
            AoidYeildAbnormalTrendVo aoidYeildAbnormalTrendVo = new AoidYeildAbnormalTrendVo();
            aoidYeildAbnormalTrendVo.setProdDate(aoidYeildStatDayVo.getProdDate().format(dfd));
            //根据时间查询上个有效自然日的数据
            AoidYeildStatMonthVo  aoidDailyYieldErrorGtsDay =  aoidLiquidAbnormalMapper.selectAoidDailyYieldAbnormalLastDay(wellids,aoidYeildStatDayVo.getProdDate().toLocalDate());
            //当日异常数量
            double  abnormalTotal = 0;
            if (aoidYeildStatDayVo.getAbnormalTotal()!=null){
                abnormalTotal=aoidYeildStatDayVo.getAbnormalTotal();
            }

            //获取一月内上个正常日错误工图数量
            double  lastDayTotal = 0;
            if(Objects.nonNull(aoidDailyYieldErrorGtsDay)){
                lastDayTotal = aoidDailyYieldErrorGtsDay.getAbnormalTotal();
            }
            //当日产量与上个正常日比(0.一致1.下降2.上升)
            double v1 = abnormalTotal / (abnormalTotal + lastDayTotal) - lastDayTotal / (abnormalTotal + lastDayTotal);
            if (abnormalTotal<lastDayTotal){
                aoidYeildAbnormalTrendVo.setLastDayAbnormalCountCompare(1);
                aoidYeildAbnormalTrendVo.setLastDayAbnormalCountProduction(Math.abs(round2(v1)));
            }else if (abnormalTotal>lastDayTotal){
                aoidYeildAbnormalTrendVo.setLastDayAbnormalCountCompare(2);
                aoidYeildAbnormalTrendVo.setLastDayAbnormalCountProduction(Math.abs(round2(v1)));
            }else {
                aoidYeildAbnormalTrendVo.setLastDayAbnormalCountCompare(0);
            }
            aoidYeildAbnormalTrendVo.setAbnormalCount(abnormalTotal);
            aoidYeildAbnormalTrendVos.add(aoidYeildAbnormalTrendVo);
        }
        return aoidYeildAbnormalTrendVos;
    }

    @Override
    public List<AoidYeildProportionBarVo> getAoidDailyYieldStatErrorGtsBar(AoidYeildVo aoidYeildVo) {
        List<AoidYeildProportionBarVo> aoidYeildProportionBarVos = new ArrayList<>();
        //通过orgId 获取子集OrgId 通过orgId获取数据
        if (aoidYeildVo.getOrg()!=null){
            log.info("开始调用通过orgId 获取子集OrgId----------------------------------");
            List<Map<String, String>> orgIds = getOrgIds(aoidYeildVo.getOrg());
            log.info("通过orgId 获取子集OrgId----------------------------------",JSONObject.toJSONString(orgIds));
            /*String map = "[{\"org_id\":\"6\",\"org_level\":3,\"org_name\":\"哈得逊零采\"},{\"org_id\":\"5\",\"org_level\":3,\"org_name\":\"哈得4  CⅢ\"},{\"org_id\":\"4\",\"org_level\":3,\"org_name\":\"哈得10  CⅠ\"},{\"org_id\":\"3\",\"org_level\":3,\"org_name\":\"哈得1  CⅠ\"}]";
            //List<Map<String, String>> orgIds = JSONObject.parseObject(map, List.class);
            List<Map<String, String>> orgIds =null;*/
            if (orgIds!=null && orgIds.size()>0){
                for (Map<String, String> orgId : orgIds) {
                    AoidYeildProportionBarVo aoidYeildProportionBarVo = new AoidYeildProportionBarVo();
                    aoidYeildProportionBarVo.setType(1);
                    if (orgId.get("org_name")!=null && orgId.get("org_id")!=null){
                        String org_name = orgId.get("org_name");
                        String org_id = orgId.get("org_id");
                        if (!org_name.equals("全部")){
                            aoidYeildProportionBarVo.setOrgId(org_id);
                            aoidYeildProportionBarVo.setOrgName(org_name);
                        }
                         List<String> wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
                        /*List<String> wellids = new ArrayList<>();
                        wellids.add("WELLT");
                        wellids.add("WELLTL100016432");
                        wellids.add("WELLTL100017249");*/
                        aoidYeildVo.setStartDate(aoidYeildVo.getStartDate() + " 00:00:00");
                        aoidYeildVo.setEndDate(aoidYeildVo.getEndDate() + " 00:00:00");
                        aoidYeildVo.setWellIdList(wellids);
                        List<AoidYeildStatDayVo>  aoidDailyYieldDayList =  aoidDailyYieldMapper.selectAoidDailyYieldErrorGtsDay(aoidYeildVo);
                        //当日错误工图数
                        double  errorgtsTotal = 0;
                        for (AoidYeildStatDayVo aoidYeildStatDayVo : aoidDailyYieldDayList) {
                            if (aoidYeildStatDayVo.getErrorgtsTotal()!=null){
                                errorgtsTotal+=aoidYeildStatDayVo.getErrorgtsTotal();
                            }
                        }
                        aoidYeildProportionBarVo.setCount(errorgtsTotal);
                        aoidYeildProportionBarVos.add(aoidYeildProportionBarVo);
                    }
                }
            }else {
             //org下无目录判断是否为末级是否包含井
                 List<String> wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
                /*List<String> wellids = new ArrayList<>();
                wellids.add("WELLT");
                wellids.add("WELLTL100016432");
                wellids.add("WELLTL100017249");*/
                aoidYeildVo.setStartDate(aoidYeildVo.getStartDate() + " 00:00:00");
                aoidYeildVo.setEndDate(aoidYeildVo.getEndDate() + " 00:00:00");
                aoidYeildVo.setWellIdList(wellids);
                List<AoidYeildStatDayVo>  aoidDailyYieldDayList =  aoidDailyYieldMapper.selectAoidDailyYieldErrorGtsGroupByWellName(aoidYeildVo);
                for (AoidYeildStatDayVo aoidYeildStatDayVo : aoidDailyYieldDayList) {
                    AoidYeildProportionBarVo aoidYeildProportionBarVo = new AoidYeildProportionBarVo();
                    aoidYeildProportionBarVo.setType(2);
                    aoidYeildProportionBarVo.setWellId(aoidYeildStatDayVo.getWellId());
                    aoidYeildProportionBarVo.setWellName(aoidYeildStatDayVo.getWellName());
                    aoidYeildProportionBarVo.setCount(aoidYeildStatDayVo.getErrorgtsTotal());
                    aoidYeildProportionBarVos.add(aoidYeildProportionBarVo);
                }
            }
        }
        return aoidYeildProportionBarVos;
    }

    @Override
    public List<AoidYeildProportionBarVo> getAoidDailyYieldAbnormalBar(AoidYeildVo aoidYeildVo) {
        List<AoidYeildProportionBarVo> aoidYeildProportionBarVos = new ArrayList<>();
        //通过orgId 获取子集OrgId 通过orgId获取数据
        if (aoidYeildVo.getOrg()!=null){
             List<Map<String, String>> orgIds = getOrgIds(aoidYeildVo.getOrg());
            //String map = "[{\"org_id\":\"6\",\"org_level\":3,\"org_name\":\"哈得逊零采\"},{\"org_id\":\"5\",\"org_level\":3,\"org_name\":\"哈得4  CⅢ\"},{\"org_id\":\"4\",\"org_level\":3,\"org_name\":\"哈得10  CⅠ\"},{\"org_id\":\"3\",\"org_level\":3,\"org_name\":\"哈得1  CⅠ\"}]";
            //List<Map<String, String>> orgIds = JSONObject.parseObject(map, List.class);
            //List<Map<String, String>> orgIds =null;
            if (orgIds!=null && orgIds.size()>0){
                for (Map<String, String> orgId : orgIds) {
                    AoidYeildProportionBarVo aoidYeildProportionBarVo = new AoidYeildProportionBarVo();
                    aoidYeildProportionBarVo.setType(1);
                    if (orgId.get("org_name")!=null && orgId.get("org_id")!=null){
                        String org_name = orgId.get("org_name");
                        String org_id = orgId.get("org_id");
                        if (!org_name.equals("全部")){
                            aoidYeildProportionBarVo.setOrgId(org_id);
                            aoidYeildProportionBarVo.setOrgName(org_name);
                        }
                         List<String> wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
                        /*List<String> wellids = new ArrayList<>();
                        wellids.add("WELLT");
                        wellids.add("WELLTL100000492");
                        wellids.add("WELLTL100000492");*/
                        aoidYeildVo.setStartDate(aoidYeildVo.getStartDate() + " 00:00:00");
                        aoidYeildVo.setEndDate(aoidYeildVo.getEndDate() + " 00:00:00");
                        aoidYeildVo.setWellIdList(wellids);
                        List<AoidYeildStatDayVo>  aoidDailyYieldDayList =  aoidLiquidAbnormalMapper.selectAoidDailyYieldAbnormalDay(aoidYeildVo);
                        //当日错误工图数
                        double  abnormalTotal = 0;
                        for (AoidYeildStatDayVo aoidYeildStatDayVo : aoidDailyYieldDayList) {
                            if (aoidYeildStatDayVo.getAbnormalTotal()!=null){
                                abnormalTotal+=aoidYeildStatDayVo.getAbnormalTotal();
                            }
                        }
                        aoidYeildProportionBarVo.setCount(abnormalTotal);
                        aoidYeildProportionBarVos.add(aoidYeildProportionBarVo);
                    }
                }
            }else {
                //org下无目录判断是否为末级是否包含井
                 List<String> wellids = getWellIds(aoidYeildVo.getOrg(), aoidYeildVo.getWellId());
             /*   List<String> wellids = new ArrayList<>();
                wellids.add("WELLT");
                wellids.add("WELLTL100001423");
                wellids.add("WELLTL100000492");*/
                aoidYeildVo.setStartDate(aoidYeildVo.getStartDate() + " 00:00:00");
                aoidYeildVo.setEndDate(aoidYeildVo.getEndDate() + " 00:00:00");
                aoidYeildVo.setWellIdList(wellids);
                List<AoidYeildStatDayVo>  aoidDailyYieldDayList =  aoidLiquidAbnormalMapper.selectAoidDailyYieldAbnormalGroupByWellName(aoidYeildVo);
                for (AoidYeildStatDayVo aoidYeildStatDayVo : aoidDailyYieldDayList) {
                    if (aoidYeildStatDayVo.getWellId()!=null){
                        LambdaQueryWrapper<AoidDailyYield> queryWrapper = new LambdaQueryWrapper<>();
                        queryWrapper.eq(AoidDailyYield::getWellId,aoidYeildStatDayVo.getWellId());
                        queryWrapper.last("limit 1");
                        AoidDailyYield dailyYield = aoidDailyYieldMapper.selectOne(queryWrapper);
                        if (Objects.nonNull(dailyYield)){
                            AoidYeildProportionBarVo aoidYeildProportionBarVo = new AoidYeildProportionBarVo();
                            aoidYeildProportionBarVo.setType(2);
                            aoidYeildProportionBarVo.setWellId(aoidYeildStatDayVo.getWellId());
                            aoidYeildProportionBarVo.setWellName(dailyYield.getWellCommonName());
                            aoidYeildProportionBarVo.setCount(aoidYeildStatDayVo.getAbnormalTotal());
                            aoidYeildProportionBarVos.add(aoidYeildProportionBarVo);
                        }
                    }
                }
            }
        }
        return aoidYeildProportionBarVos;
    }
    public List<String> getWellIds(String org, String wellId) {
        if (Objects.nonNull(wellId) && !wellId.isEmpty()) return new ArrayList<>(Arrays.asList(new String[]{wellId}));

        if (Objects.isNull(org) || org.isEmpty()) return null;

        String resultStr = baseDataClient.getWellListByOrgID(org);
        if (Objects.isNull(resultStr) || resultStr.isEmpty()) return null;

        Result result = JSONObject.parseObject(resultStr, Result.class);
        if (Objects.isNull(result) || Objects.isNull(result.getData()))
            return null;
        JSONArray array = (JSONArray) result.getData();
        ArrayList<String> re = new ArrayList<>();
        for (int i = 0; i < array.size(); i++) {
            re.add(array.getJSONObject(i).getString("id"));
        }
        return re;
    }
    public List<Map<String, String>> getOrgIds(String org) {
        List<Map<String, String>> list = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getOrganization(org), Result.class).getData().toString(), List.class);
        return list;
    }
}
