package com.gct.aoid.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.aoid.entity.AoidGkHeavyModel;
import com.gct.aoid.entity.AoidSinglewellheavyConfig;
import com.gct.aoid.mapper.AoidGkHeavyModelMapper;
import com.gct.aoid.service.AoidGkHeavyModelService;
import com.gct.common.core.result.CodeMsg;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Lloyd
 * @since 2021-01-31
 */
@Service
public class AoidGkHeavyModelServiceImpl extends ServiceImpl<AoidGkHeavyModelMapper, AoidGkHeavyModel> implements AoidGkHeavyModelService {
    @Autowired
    AoidGkHeavyModelMapper aoidGkHeavyModelMapper;

    //region 第一种方式实现:自定义缓存对象

    private Map<String, AoidGkHeavyModel> allGKModels = new HashMap<>();
    private Object lock = new Object();


    /**
     * 查询稠油诊断结果
     * @return
     */
    public Result  selectGKresult()
    {
        QueryWrapper<AoidGkHeavyModel> queryWrapper = new QueryWrapper<>();
        QueryWrapper<AoidGkHeavyModel> wrapper = queryWrapper.select(
                "result",
                "hid");
        List<Map<String, Object>> maps = baseMapper.selectMaps(wrapper);
        HashMap<String, Object> allMap = new HashMap<>();
        allMap.put("result","全部");
        allMap.put("hid","");
        maps.add(0,allMap);
        return Result.ok(maps);
    }




    /*
    根据工况查询稠油工况模型
     */
    @Override
    public AoidGkHeavyModel getGKModel_HeavyByResultType(String result) {
        if (allGKModels == null || allGKModels.size() == 0 || !allGKModels.containsKey(result))
            synchronized (lock) {
                if (allGKModels == null || allGKModels.size() == 0 || !allGKModels.containsKey(result)) {
                    List<AoidGkHeavyModel> gkModels = aoidGkHeavyModelMapper.selectList(new QueryWrapper<>());
                    if (gkModels == null || gkModels.size() == 0) return null;
                    allGKModels.clear();
                    for (AoidGkHeavyModel gkModel : gkModels) {
                        allGKModels.put(gkModel.getResult(), gkModel);
                    }
                }
            }
        if (allGKModels.containsKey(result))
            return allGKModels.get(result);
        return null;
    }


    /**
     *根据id查询稠油工况模型
     * @param hid
     * @return
     */
    @Override
    public Result selectOneByGKModel_HeavyByResultType(double hid){
        AoidGkHeavyModel aoidGkHeavyModel = aoidGkHeavyModelMapper.selectOneByGKModel_HeavyByResultType(hid);
        return Result.ok(aoidGkHeavyModel);
    }

    /**
     *根据工况查询稠油工况模型
     * @param result
     * @return
     */
    @Override
    public Result selectOneByGKModel_HeavyByResultType1(String result){
        AoidGkHeavyModel aoidGkHeavyModel = aoidGkHeavyModelMapper.selectOneByGKModel_HeavyByResultType1(result);
        return Result.ok(aoidGkHeavyModel);
    }


    /**
     * 添加工况模型数据
     * @param aoidGkHeavyModel
     * @return
     */
    @Override
    public Result addAoidGkHeavyModel(AoidGkHeavyModel aoidGkHeavyModel)
    {
        if(aoidGkHeavyModel.getResult() != null) {
            if(this.selectOneByGKModel_HeavyByResultType1(aoidGkHeavyModel.getResult()).getData() != null) {
                return Result.create(CodeMsg.createCodeMsg(-1,"该工况结果已存在"));
            }
        }
        //添加单井属性数据
        int insert = aoidGkHeavyModelMapper.insert(aoidGkHeavyModel);
        return insert==1?Result.create(CodeMsgEnum.ADD_SUCCESS):Result.create(CodeMsgEnum.ADD_FAIL);
    }

    /**
     * 修改工况模型数据
     * @param aoidGkHeavyModel
     * @return
     */
    public Result updateAoidGkHeavyModel(AoidGkHeavyModel aoidGkHeavyModel)
    {
        int update = aoidGkHeavyModelMapper.updateById(aoidGkHeavyModel);
        return update==1?Result.create(CodeMsgEnum.MODIFY_SUCCESS):Result.create(CodeMsgEnum.MODIFY_FAIL);
    }


    /**
     * 根据工况结果查询模型相关数据
     * @param pageNum
     * @param pageSize
     * @param result
     * @return
     */
    public Result findAllAoidGkHeavyModel(Integer pageNum, Integer pageSize,double result)
    {

        Page<AoidGkHeavyModel> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        IPage<AoidGkHeavyModel> page = aoidGkHeavyModelMapper.findAllAoidGkHeavyModel
                (paramPage,  result);
        return Result.ok(page);

    }


}
