package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.gct.aoid.algorithm.*;
import com.gct.aoid.entity.*;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.service.*;
import com.gct.common.core.result.Result;
import com.gct.common.util.SGTUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;

import java.lang.reflect.InvocationTargetException;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * class name: CalDiagramProductionService
 * @explain 量液服务类
 * @author lloyd
 * @version 1.0
 * @since 2021/1/29 下午9:47
 */
@Component
@Slf4j
public class YieldErrorAnalysisCalculationService {

    @Autowired
    private BaseDataClient baseDataClient;

    @Autowired
    private AoidDailyYieldService aoidDailyYieldService;

    @Autowired
    private AoidSinglegtYieldService aoidSinglegtYieldService;

    @Autowired
    private YieldErrorAnalysisService yieldErrorAnalysisService;

    private final DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private final DateTimeFormatter dfValue = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    

    /**
     *误差分析定时运行
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchMethodException
     */
    public void calYieldError(String dateStart,String dateEnd) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException, ParseException {

        String baseParam = baseDataClient.getjhAllYieldError();
        Result result = JSONObject.parseObject(baseParam, Result.class);
        Object data = result.getData();
        String dateTime = null;
        if (dateStart==null&&dateEnd==null)
        {
            dateTime = LocalDateTime.now().minusDays(1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))+" 00:00:00";
            calYieldErrorEnd(data,dateTime);
        }
        else {
            List<String> dateValue = findDates(dateStart,dateEnd);
            for (int i=0;i<dateValue.size();i++)
            {
                calYieldErrorEnd(data, dateValue.get(i)+" 00:00:00");
            }

        }

    }


    /**
     *误差分析定时运行
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchMethodException
     */
    public void calYieldErrorEnd(Object data,String dateTime) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException, ParseException {

        double systemYieldRate =0.0;
        double a2YieldRate =0.0;
        String isExpectValue =null;
        double areaRate =0.0;

        JSONArray jsonArray = JSONArray.parseArray(data.toString());
        for (int i = 0; i < jsonArray.size(); i++) {


            JSONObject jo = jsonArray.getJSONObject(i);

            String wellId = jo.getString("wellId");
            String[] stationIds = jo.getString("stationId").split("@");
            String zyq = stationIds[0];
            double ErrorEnd =0.0;

            A2Basedata a2Today = getA2Basedata(wellId, LocalDateTime.parse(dateTime,df).plusDays(1).format(df));
            List<AoidDailyYield> aoidDailyYieldTodayValue = aoidDailyYieldService.findByJhAoidDailyA2(wellId,dateTime);



            if (aoidDailyYieldTodayValue.size()>0)
            {
                try {
                    Thread.sleep(1000);
                    if (a2Today.LiqProdDaily != null) {
                        if (a2Today.LiqProdDaily != 0)
                        {
                            if (aoidDailyYieldTodayValue.get(0).getLiqProdDaily()!=null)
                            {
                                YieldErrorAnalysis yieldErrorAnalysis = new YieldErrorAnalysis();
                                double systemLiqProdDaily = aoidDailyYieldTodayValue.get(0).getLiqProdDaily();
                                double difYiele = Math.abs(systemLiqProdDaily-a2Today.LiqProdDaily);
                                double ErrorValue = round2(Math.abs((systemLiqProdDaily-a2Today.LiqProdDaily)/a2Today.LiqProdDaily));
                                yieldErrorAnalysis.setWellId(wellId);
                                yieldErrorAnalysis.setZyq(zyq);
                                yieldErrorAnalysis.setProdDate(LocalDateTime.parse(dateTime,df));
                                yieldErrorAnalysis.setTodayLiqProdDaily(systemLiqProdDaily);
                                yieldErrorAnalysis.setTodayA2LiqProdDaily(a2Today.LiqProdDaily);
                                yieldErrorAnalysis.setTodayOilPressure(a2Today.yy);
                                yieldErrorAnalysis.setTodayCasingPressure(a2Today.ty);
                                if (systemLiqProdDaily<=3 && a2Today.LiqProdDaily<=3)
                                {
                                    if(difYiele == 0)
                                    {
                                        ErrorEnd =0;
                                    }
                                    else if (difYiele<=0.8)
                                    {
                                        ErrorEnd =0.1;
                                    }
                                    else if (difYiele<=1.5)
                                    {
                                        ErrorEnd =0.2;
                                    }
                                }
                                else if (ErrorValue>1.1 && difYiele<=1.5) {
                                    ErrorEnd =0.2;
                                }
                                else if (ErrorValue>1.1 && difYiele<=5) {
                                    ErrorEnd =0.99;
                                }
                                else if (ErrorValue>1.1 && difYiele<=10) {
                                    ErrorEnd =1.25;
                                }
                                else if (ErrorValue>1.1 && difYiele>=10) {
                                    ErrorEnd =1.45;
                                }
                                else {
                                    ErrorEnd = ErrorValue;
                                }


                                yieldErrorAnalysis.setTodayYieldError(ErrorEnd);
                                yieldErrorAnalysis.setTodayRemark(a2Today.remarks);


                                String realDate = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))+" 00:00:00";

                                String newUpdateTime = baseDataClient.getCbPcDevWellAttrMaxUpdateTime(wellId,realDate,"1");
                                Result UpdateTime  = JSONObject.parseObject(newUpdateTime, Result.class);
                                Object dataUpdate = UpdateTime.getData();

                                if (dataUpdate!=null) {
                                    JSONObject jsonArrayUpdate = JSONObject.parseObject(dataUpdate.toString());
                                    String updateNew = jsonArrayUpdate.getString("updateTime").split("\\.")[0];
                                    if (updateNew.length()==19)
                                    {
                                        yieldErrorAnalysis.setCalibrationNew(LocalDateTime.parse(updateNew,df));
                                    }
                                }
                                yieldErrorAnalysis.setSummaryDescription(aoidDailyYieldTodayValue.get(0).getSummaryDescription());
                                yieldErrorAnalysis.setIsExpect(isExpectValue);

                                if (!yieldErrorAnalysisService.update(yieldErrorAnalysis, new LambdaUpdateWrapper<YieldErrorAnalysis>()
                                        .eq(YieldErrorAnalysis::getWellId, yieldErrorAnalysis.getWellId())
                                        .eq(YieldErrorAnalysis::getProdDate, yieldErrorAnalysis.getProdDate())))
                                    yieldErrorAnalysisService.save(yieldErrorAnalysis);
                            }
                        }
                    }

                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }



        }


    }




    //获取A2基础数据
    public A2Basedata getA2Basedata(String wellId, String A2ProdDate)throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException
    {

        A2Basedata a2Basedata = new A2Basedata();
        ProDataForCalProduction a2DataVolYesterday;
        ProDataForCalProduction a2DataStatusYesterday;


        String a2dataStrVolYesterday = baseDataClient.getA2VolDataForCalProduction(wellId, A2ProdDate);
        a2DataVolYesterday = (ProDataForCalProduction) JSONObject.parseObject(a2dataStrVolYesterday, Result.class).toFiguredOutObject(ProDataForCalProduction.class);

        String a2dataStrYesterday = baseDataClient.getA2StatusDataForCalProduction(wellId, A2ProdDate);
        a2DataStatusYesterday = (ProDataForCalProduction) JSONObject.parseObject(a2dataStrYesterday, Result.class).toFiguredOutObject(ProDataForCalProduction.class);


        if (a2DataVolYesterday!=null)
        {
            a2Basedata.LiqProdDaily = (a2DataVolYesterday.getLiqProdDaily()==null)?0:a2DataVolYesterday.getLiqProdDaily();
            a2Basedata.yy = (a2DataStatusYesterday.getTubingPres()==null)?0:a2DataStatusYesterday.getTubingPres();
            a2Basedata.ty = (a2DataStatusYesterday.getCasingPres()==null)?0:a2DataStatusYesterday.getCasingPres();
            a2Basedata.remarks = a2DataVolYesterday.getRemarks();
        }


        return a2Basedata;
    }




    //A2基础数据类
    public class A2Basedata
    {
        private Double LiqProdDaily;
        private Double yy;
        private Double ty;
        private  String remarks;
    }


    public static double round2(double in) {
        DecimalFormat df   = new DecimalFormat("######0.00");
        return Double.valueOf(df.format(in));
    }


    /**
     * 获取时间范围
     * @param dBegin
     * @param dEnd
     * @return
     * @throws ParseException
     * @throws java.text.ParseException
     */
    public static List findDates(String dBegin, String dEnd) throws ParseException, java.text.ParseException {

//日期工具类准备

        DateFormat format = new SimpleDateFormat("yyyy-MM-dd");

//设置开始时间

        Calendar calBegin = Calendar.getInstance();

        calBegin.setTime(format.parse(dBegin));

//设置结束时间

        Calendar calEnd = Calendar.getInstance();

        calEnd.setTime(format.parse(dEnd));

//装返回的日期集合容器

        List Datelist = new ArrayList();

//将第一个月添加里面去

        Datelist.add(format.format(calBegin.getTime()));

// 每次循环给calBegin日期加一天，直到calBegin.getTime()时间等于dEnd

        while (format.parse(dEnd).after(calBegin.getTime())) {

// 根据日历的规则，为给定的日历字段添加或减去指定的时间量

            calBegin.add(Calendar.DAY_OF_MONTH, 1);

            Datelist.add(format.format(calBegin.getTime()));

        }


        return Datelist;

    }





}
